﻿package compo_ritual
{
    import flash.display.Sprite;
    import flash.media.Sound;
    import flash.ui.Keyboard;
    import meyermike.collision.CollisionInfo;
    import meyermike.constants.Colors;
    import meyermike.core.GameObject;
    import meyermike.InputUtil;
    import meyermike.PlatformerCharacter;
    import meyermike.Sounds;
    import meyermike.util.spriteanim.SpriteAnimUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PlatformMan extends PlatformerCharacter
    {
        [Embed(source='assets/platformman/idle.png')]
        private static const IDLE:Class;
        [Embed(source='assets/platformman/blink.png')]
        private static const BLINK:Class;
        private static const IDLE_FRAMES:Array = [IDLE, IDLE, IDLE, IDLE, IDLE, IDLE, IDLE, BLINK];

        [Embed(source='assets/platformman/walk0.png')]
        private static const WALK0:Class;
        [Embed(source='assets/platformman/walk2.png')]
        private static const WALK2:Class;
        private static const RUN_FRAMES:Array = [WALK0, IDLE, WALK2, IDLE];

        [Embed(source='assets/platformman/skid0.png')]
        private static const SKID0:Class;
        [Embed(source='assets/platformman/skid1.png')]
        private static const SKID1:Class;
        private static const SKID_FRAMES:Array = [SKID0, SKID1];

        [Embed(source='assets/platformman/skidturn0.png')]
        private static const SKIDTURN0:Class;
        [Embed(source='assets/platformman/skidturn1.png')]
        private static const SKIDTURN1:Class;
        private static const SKIDTURN_FRAMES:Array = [SKIDTURN0, SKIDTURN1];

        [Embed(source='assets/platformman/jump0.png')]
        private static const JUMP0:Class;
        [Embed(source='assets/platformman/jump1.png')]
        private static const JUMP1:Class;
        private static const JUMP_FRAMES:Array = [JUMP0, JUMP1];

        [Embed(source='assets/platformman/fall0.png')]
        private static const FALL0:Class;
        [Embed(source='assets/platformman/fall1.png')]
        private static const FALL1:Class;
        private static const FALL_FRAMES:Array = [FALL0, FALL1];

        [Embed(source='assets/platformman/lose.png')]
        private static const LOSE:Class;
        private static const LOSE_FRAMES:Array = [LOSE];

        [Embed(source='assets/platformman/won1.png')]
        private static const WON0:Class;
        [Embed(source='assets/platformman/won2.png')]
        private static const WON1:Class;
        private static const WON_FRAMES:Array = [WON0, WON1];

        private static var SoundChannel:flash.media.SoundChannel;

        [Embed(source='assets/jump1.mp3')]
        private static const Jump1SoundClass:Class;
        private static var Jump1Sound:Sound = new Jump1SoundClass();

        [Embed(source='assets/jump2.mp3')]
        private static const Jump2SoundClass:Class;
        private static var Jump2Sound:Sound = new Jump2SoundClass();

        [Embed(source='assets/jump3.mp3')]
        private static const Jump3SoundClass:Class;
        private static var Jump3Sound:Sound = new Jump3SoundClass();

        [Embed(source='assets/jump4.mp3')]
        private static const Jump4SoundClass:Class;
        private static var Jump4Sound:Sound = new Jump4SoundClass();

        private var mAnimUtil:SpriteAnimUtil;

        private static const MOVE_RIGHT_KEY:Number = Keyboard.RIGHT;
        private static const MOVE_LEFT_KEY:Number = Keyboard.LEFT;
        private static const NUM_AIR_JUMPS:Number = 3;
        private static const PLAYER_JUMP_SPEED:Number = 600;
        private static const PLAYER_JUMP_STOP_ACCEL:Number = 10000;

        private static const SKID_START_THRESHOLD:Number = 200.0;
        private static const SKID_STOP_THRESHOLD:Number = 1.0;

        private var mAirJumpsRemaining:Number = NUM_AIR_JUMPS;

        private static const TIME_BETWEEN_FOOTSTEPS:Number = 0.125;
        private var mFootStepTimer:Number;

        // keys you can use to jump
        private static const JUMP_KEYS:Array = [Keyboard.SPACE, Keyboard.SHIFT, Keyboard.UP, 90];

        // remember which key they used to jump, so we know which key's release event we care about
        private var mJumpKey:uint = 0;

        private var mIsJumping:Boolean = false;
        public function get IsJumping():Boolean { return mIsJumping; }
        public function set IsJumping(value:Boolean):void { mIsJumping = value; }

        public function get skidding():Boolean { return -1 != mAnimUtil.curAnim.indexOf("skid");  }


        public override function Init():void
        {
            super.Init();

            mWalkAccel = 400;
            mAirAccel = 500;
            mStopAccel = 2000;
            mMaxWalkSpeed = 400;

            CreateGraphics();
            Restart();
        }

        protected function CreateGraphics():void
        {
            if (null == mAnimUtil)
            {
                mAnimUtil = new SpriteAnimUtil(this);
                mAnimUtil.scaleX = 2.0;
                mAnimUtil.scaleY = 2.0;
                mAnimUtil.addAnimation("idle", IDLE_FRAMES, 4.0);
                mAnimUtil.addAnimation("run", RUN_FRAMES);
                mAnimUtil.addAnimation("skid", SKID_FRAMES, 16.0);
                mAnimUtil.addAnimation("skidturn", SKIDTURN_FRAMES, 32.0);
                mAnimUtil.addAnimation("jump", JUMP_FRAMES);
                mAnimUtil.addAnimation("fall", FALL_FRAMES);
                mAnimUtil.addAnimation("lose", LOSE_FRAMES);
                mAnimUtil.addAnimation("won", WON_FRAMES);
            }
        }

        /**
         * per-frame update
         * @param   elapsedTime - time since the last frame
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);
            mAnimUtil.update(elapsedTime);

            if (PhysicsEnabled)
            {
                Walk(elapsedTime);
                Jumping(elapsedTime);
            }
        }

        /**
         * change dx based on user input
         */
        protected function Walk(elapsedTime:Number):void
        {
            var desiredDirection:Number = STOP;
            if (InputUtil.IsDown(MOVE_RIGHT_KEY))
            {
                desiredDirection = RIGHT;
            }
            else if (InputUtil.IsDown(MOVE_LEFT_KEY))
            {
                desiredDirection = LEFT;
            }

            WalkAccel(desiredDirection, elapsedTime);


            // update ground animations now
            if (IsOnGround)
            {
                if ( STOP == desiredDirection )
                {
                    if ( Math.abs(dx) > SKID_START_THRESHOLD)
                    {
                        if (!skidding)
                        {
                            //GameContent.PlaySound_Skid();
                        }
                        mAnimUtil.curAnim = "skid";
                    }
                    else if ( skidding && (Math.abs(dx) > SKID_STOP_THRESHOLD) )
                    {
                        // just keep skidding
                    }
                    else
                    {
                        mAnimUtil.curAnim = "idle";
                    }
                }
                else
                {
                    if ( ( ( RIGHT == desiredDirection ) && ( dx < 0.0 ) )
                            || ( ( LEFT == desiredDirection ) && ( dx > 0.0 ) ) )
                    {
                        if (!skidding)
                        {
                            //GameContent.PlaySound_Skid();
                        }
                        mAnimUtil.curAnim = "skidturn";
                    }
                    else
                    {
                        mAnimUtil.curAnim = "run";
                    }
                }
            }

            Footsteps(elapsedTime);
        }

        /**
         * take care of making footstep noises when walking
         */
        private function Footsteps(elapsedTime:Number):void
        {
            if ("run" == mAnimUtil.curAnim)
            {
                mFootStepTimer += elapsedTime;
                if (mFootStepTimer >= TIME_BETWEEN_FOOTSTEPS)
                {
                    mFootStepTimer = 0;
                    GameContent.PlaySound_Footstep();
                }
            }
        }

        /**
         *
         */
        protected function Jumping(elapsedTime:Number):void
        {
            if ( InputUtil.IsUp(mJumpKey) || IsOnGround)
            {
                this.IsJumping = false;
                mJumpKey = 0;
            }

            // if we aren't holding the jump button anymore, cut
            // off the jump height
            if (!IsOnGround && !this.IsJumping && (this.dy < 0) )
            {
                this.dy += PLAYER_JUMP_STOP_ACCEL * elapsedTime;

                // don't shoot us down, though
                if (this.dy > 0)
                {
                    this.dy = 0;
                }
            }

            // starting a new jump
            var justPressedJumpKey:uint = 0;
            for each (var keyCode:uint in JUMP_KEYS)
            {
                if (InputUtil.WasJustPressed(keyCode))
                {
                    justPressedJumpKey = keyCode;
                }
            }

            if (InputUtil.WasJustPressed(justPressedJumpKey))
            {
                if (IsOnGround || (mAirJumpsRemaining > 0) )
                {
                    this.dy = -PLAYER_JUMP_SPEED;
                    this.IsJumping = true;
                    mJumpKey = justPressedJumpKey;

                    SoundChannel = Sounds.PlaySound(Jump1Sound, SoundChannel, 0.5);
                }

                if (!IsOnGround)
                {
                    if (NUM_AIR_JUMPS == mAirJumpsRemaining)
                    {
                        SoundChannel = Sounds.PlaySound(Jump2Sound, SoundChannel, 0.5);
                    }
                    else if ( (NUM_AIR_JUMPS-1) == mAirJumpsRemaining)
                    {
                        SoundChannel = Sounds.PlaySound(Jump3Sound, SoundChannel, 0.5);
                    }
                    else if ( (NUM_AIR_JUMPS-2) == mAirJumpsRemaining)
                    {
                        SoundChannel = Sounds.PlaySound(Jump4Sound, SoundChannel, 0.5);
                    }

                    --mAirJumpsRemaining;
                }
            }

            // reset air jumps after landing
            if (IsOnGround)
            {
                mAirJumpsRemaining = NUM_AIR_JUMPS;
            }
            else // jump animations!
            {
                mAnimUtil.curAnim = ( dy < 0 ) ? "jump" : "fall";
            }
        }

        /**
         * Get it ready to be reused again
         */
        protected override function Restart():void
        {
            super.Restart();
            mAnimUtil.curAnim = "idle";
            mFootStepTimer = 0;
        }

        /**
         * override to let yourself go through things
         */
        public override function CanPassThrough(ci:CollisionInfo, object:GameObject):Boolean
        {
            return (object is Item);
        }

        /**
         *
         */
        public function DoLoseAnimation():void
        {
            mAnimUtil.curAnim = "lose";
        }

        /**
         *
         */
        public function DoWinAnimation():void
        {
            mAnimUtil.curAnim = "won";
        }


        public function ResetAirJumps():void
        {
            mAirJumpsRemaining = NUM_AIR_JUMPS;
        }
    }
}
