﻿package compo_ritual
{
    import compo_ritual.gamestates.Game;
    import compo_ritual.gamestates.TitleScreen;
    import compo_ritual.gamestates.YouWinScreen;
    import flash.events.Event;
    import flash.events.EventDispatcher;
    import flash.geom.Matrix;
    import meyermike.collision.CollisionEvent;
    import meyermike.constants.Colors;
    import meyermike.core.GameInterface;
    import meyermike.core.GameObject;
    import meyermike.script.Step;
    import meyermike.script.step.Loop;
    import meyermike.script.step.RunFunc;
    import meyermike.script.step.WaitTime;
    import meyermike.Settings;
    import meyermike.widget.DisappearingText;
    import meyermike.widget.TextField;

    /**
     * ...
     * @author Mike Meyer
     */
    public class RitualLevels extends EventDispatcher
    {
        public static const EVENT_WIN:String = "win";
        public static const EVENT_LOSE:String = "lose";

        private static const TILE_SIZE:Number = 32;
        private static const DEATH_BOUNCE:Number = -500;

        private var game:GameInterface;
        private var statusText:TextField;
        private var levelMessage:TextField;
        private var curLevel:uint;
        private var levels:Array;

        private var item1:Item;
        private var item2:Item;
        private var item3:Item;

        public function get player():PlatformMan { return _player; }
        private var _player:PlatformMan;

        private static const LEVEL_STATE_1:uint = 0;
        private static const LEVEL_STATE_2:uint = 1;
        private static const LEVEL_STATE_3:uint = 2;
        private static const ALL_COLLECTED:uint = 2;
        private var levelState:uint;

        protected var script:Step;

        public function RitualLevels(game:GameInterface)
        {
            this.game = game;

            curLevel = 0;
            levels = [
                teachMovementLevel,
                teachJumpLevel,
                teachRitualPartOneLevel,
                teachRitualPartTwoLevel,
                teachDoubleJumpLevel,
                teachFourTimes,
                trickyLevel,
                beQuickLevel,
                pitLevel,
                goUnderLevel,
                tunnelLevel,
                underOverLevel,
                youWinLevel
                ];
            levelState = LEVEL_STATE_1;

            createObjects();
         }

         /// create the objects that are shared between levels
         private function createObjects():void
         {
            statusText = game.Layers.UiLayer.Add(TextField) as TextField;
            statusText.SetFontColor(Colors.WHITE);

            levelMessage = game.Layers.UiLayer.Add(TextField) as TextField;
            levelMessage.SetFontSize(24);
            levelMessage.SetFontColor(Colors.WHITE_ENOUGH);

            item1 = makeItem( GameContent.ITEM1_FRAMES );
            item2 = makeItem( GameContent.ITEM2_FRAMES );
            item3 = makeItem( GameContent.ITEM3_FRAMES );

            _player = game.Layers.PlayerLayer.Add(PlatformMan) as PlatformMan;
            _player.addEventListener(CollisionEvent.EVENT_COLLIDE, playerHitSomething);
        }

        /**
         * start the first level
         */
        public function begin():void
        {
            curLevel = 0;
            cleanUpLevel();
            initLevel();
        }

        /**
         * start the next level
         */
        public function nextLevel():void
        {
            ++curLevel;
            cleanUpLevel();
            initLevel();
        }

        /**
         * restart the current level
         */
        public function restartLevel():void
        {
            cleanUpLevel();
            initLevel();
        }

        /// clean up after old levels
        private function cleanUpLevel():void
        {
            game.Layers.PlatformLayer.RemoveAll();
            game.Layers.ItemLayer.RemoveAll();
        }

        /**
         * update the palyer status textfield
         */
        private function updateStatusText():void
        {
            statusText.SetText( "level " + (curLevel+1) + " / " + String(levels.length) );
        }


        /// set up the current level
        private function initLevel():void
        {
            updateStatusText();
            game.Time.TimeScale = 1.0;
            levelState = LEVEL_STATE_1;
            levelMessage.SetText("");
            levelMessage.SetFontColor(Colors.WHITE_ENOUGH);
            player.PhysicsEnabled = true;
            player.AffectedByGravity = true;

            statusText.CenterOn(0.5 * Settings.StageWidth, 0.97 * Settings.StageHeight);
            levelMessage.CenterOn(0.5 * Settings.StageWidth, 0.3 * Settings.StageHeight);

            game.Layers.ItemLayer.AddPremade(item1);
            item1.x = TILE_SIZE * 11.5;
            item1.y = TILE_SIZE * 12.5
            game.Layers.ItemLayer.AddPremade(item2);
            item2.x = TILE_SIZE * 13.5;
            item2.y = TILE_SIZE * 12.5;
            game.Layers.ItemLayer.AddPremade(item3);
            item3.x = TILE_SIZE * 15.5;
            item3.y = TILE_SIZE * 12.5;

            game.FollowTarget = null;
            game.CameraPosition.x = 0;
            game.CameraPosition.y = 0;

            player.dx = 0;
            player.dy = 0;
            player.rotation = 0;
            player.dr = 0;
            player.ResetAirJumps();

            if (curLevel < levels.length)
            {
                var f:Function = levels[curLevel];
                f();
            }
            else
            {
                // I guess you won!
                game.ChangeGameState(new YouWinScreen());
            }
        }

        /**
         * when the player hits certain things, we need to make stuff happen
         */
        private function playerHitSomething(e:CollisionEvent):void
        {
            if (e.object is Item)
            {
                var item:Item = e.object as Item;
                itemCollisionCheck(item);
            }
            if (e.object is Spike)
            {
                lose("");
            }
        }

        /// handle game logic for player-item collisions
        private function itemCollisionCheck(collidedWith:Item):void
        {
            if ( (LEVEL_STATE_1 == levelState)
               &&(item1 == collidedWith) )
            {
                levelState = LEVEL_STATE_2;
                GameContent.PlaySound_Collect1();
                game.Layers.ItemLayer.Remove(item1);
            }
            else if ( (LEVEL_STATE_2 == levelState)
                    &&(item2 == collidedWith) )
            {
                levelState = LEVEL_STATE_3;
                GameContent.PlaySound_Collect2();
                game.Layers.ItemLayer.Remove(item2);
            }
            else if ( (LEVEL_STATE_3 == levelState)
                    &&(item3 == collidedWith) )
            {
                levelState = ALL_COLLECTED;
                GameContent.PlaySound_Collect3();
                game.Layers.ItemLayer.Remove(item3);

                win();
            }
            else
            {
                lose("       FAILURE\nYou must collect them\n in the right order!");
            }
        }

        /**
         * the player just beat this level! handle it!
         */
        private function win():void
        {
            player.dx = 0;
            player.dy = 0;
            player.PhysicsEnabled = false;
            player.AffectedByGravity = false;

            levelMessage.SetText("LEVEL COMPLETE!\n    \\(^o^)/");
            levelMessage.SetFontColor(Colors.WHITE);

            player.DoWinAnimation();

            script = new Loop(1, [
                new WaitTime(1.5),
                new RunFunc(nextLevel) ] );

            dispatchEvent( new Event(EVENT_WIN) );
        }

        /**
         * the player just lost this level! handle it!
         */
        private function lose(loseMessage:String):void
        {
            player.dx *= 0.25;
            player.dy = DEATH_BOUNCE;
            player.PhysicsEnabled = false;

            levelMessage.SetText(loseMessage);
            levelMessage.SetFontColor(Colors.WHITE);

            player.DoLoseAnimation();
            player.dr = 360;

            script = new Loop(1, [
                new WaitTime(1.0),
                new RunFunc(restartLevel) ] );

            dispatchEvent( new Event(EVENT_LOSE) );
        }

        /**
         * Add an item to the scene
         */
        private function makeItem(frames:Array):Item
        {
            var item:Item = new Item();
            item.SetAnimation(frames, 16.0);
            return item;
        }

        /**
         * per-frame update
         */
        public function update(elapsedTime:Number):void
        {
            if (null != script)
            {
                var done:Boolean = script.Execute(elapsedTime);
                if (done)
                {
                    script = null;
                }
            }

            // check to see if something crazy happened
            if ( (player.PhysicsEnabled) && (player.y > 1000) )
            {
                lose("You fell!");
            }
        }

        ///
        private function teachMovementLevel():void
        {
            var w:Number = Settings.StageWidth;
            var h:Number = Settings.StageHeight;

            // left wall
            GameContent.addPlatform( game.Layers.PlatformLayer, 0, 0, TILE_SIZE, h );

            // right wall
            GameContent.addPlatform( game.Layers.PlatformLayer, w - TILE_SIZE, 0, TILE_SIZE, h );

            // floor
            GameContent.addPlatform( game.Layers.PlatformLayer, TILE_SIZE, h - 2*TILE_SIZE, w-(2.0*TILE_SIZE), TILE_SIZE );
            GameContent.addPlatform( game.Layers.PlatformLayer, TILE_SIZE, h -  TILE_SIZE, 10.0*TILE_SIZE, TILE_SIZE );

            // ceilings
            GameContent.addPlatform( game.Layers.PlatformLayer, TILE_SIZE, 0, w-(2.0*TILE_SIZE), (3.0*TILE_SIZE) );
            GameContent.addPlatform( game.Layers.PlatformLayer, TILE_SIZE, (6.0*TILE_SIZE) , w-(2.0*TILE_SIZE), (6.0*TILE_SIZE) );

            item1.x = TILE_SIZE * 10.5;
            item1.y = TILE_SIZE * 12.5

            item2.x = item1.x + (3 * TILE_SIZE);
            item2.y = item1.y;

            item3.x = item2.x + (3 * TILE_SIZE);
            item3.y = item1.y;

            player.x = TILE_SIZE * 3.5;
            player.y = TILE_SIZE * 12.5;

            // items

            levelMessage.CenterOn(0.5 * Settings.StageWidth, 0.3 * Settings.StageHeight);
            levelMessage.SetText("use the arrow keys\nto move left and right");

            statusText.CenterOn(14 * TILE_SIZE, 14.5 * TILE_SIZE);
        }

        ///
        private function teachJumpLevel():void
        {
            var w:Number = 5.0 * TILE_SIZE;
            var h:Number = 25.0 * TILE_SIZE;
            var secondShaftHeight:Number = 7.0 * TILE_SIZE;

            game.FollowTarget = player;

            // small ceiling
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                TILE_SIZE,
                0,
                TILE_SIZE,
                TILE_SIZE );

            // left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0,
                0,
                TILE_SIZE,
                h );

            // right wall for shaft
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                2.0 * TILE_SIZE,
                0,
                TILE_SIZE,
                h - (2 * TILE_SIZE) );

            // floor
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                TILE_SIZE,
                h - TILE_SIZE,
                w - (2.0 * TILE_SIZE),
                TILE_SIZE );

            // 2nd wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                w - TILE_SIZE,
                h - secondShaftHeight,
                TILE_SIZE,
                secondShaftHeight );

            // 2nd ceiling
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                w - 2 * TILE_SIZE,
                h - secondShaftHeight,
                TILE_SIZE,
                TILE_SIZE );

            player.x = 1.5 * TILE_SIZE;
            player.y = 1.5 * TILE_SIZE;

            item1.x = TILE_SIZE * 3.5;
            item1.y = h - (3.5 * TILE_SIZE);

            item2.x = item1.x;
            item2.y = item1.y - TILE_SIZE;

            item3.x = item2.x;
            item3.y = item2.y - TILE_SIZE;

            levelMessage.CenterOn(8.5 * TILE_SIZE, 23.5 * TILE_SIZE);
            levelMessage.SetText("press\nspace\nbar\nto\njump\n\nhold\nto\ngo\nhigher");

            statusText.CenterOn(2.5 * TILE_SIZE,  h + 1.0 * TILE_SIZE);
        }

        ///
        private function teachRitualPartOneLevel():void
        {
            var w:Number = Settings.StageWidth;
            var h:Number = Settings.StageHeight;

            // left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0,
                0,
                TILE_SIZE,
                h );

            // right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                w - TILE_SIZE,
                0,
                TILE_SIZE,
                h );

            // floor
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                TILE_SIZE,
                h - TILE_SIZE,
                6 * TILE_SIZE,
                TILE_SIZE );
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                12 * TILE_SIZE,
                h - TILE_SIZE,
                6 * TILE_SIZE,
                TILE_SIZE );

            // ceiling
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                TILE_SIZE,
                0,
                w - (2.0 * TILE_SIZE),
                TILE_SIZE );

            // obstacle-low
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                7 * TILE_SIZE,
                h - (4 * TILE_SIZE),
                5 * TILE_SIZE,
                3 * TILE_SIZE );

            // obstacle-high
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                7 * TILE_SIZE,
                6 * TILE_SIZE,
                5 * TILE_SIZE,
                1 * TILE_SIZE);

            player.x = 3.5 * TILE_SIZE;
            player.y = h - 2 * TILE_SIZE;

            item1.x = 4.5 * TILE_SIZE;
            item1.y = 11.5 * TILE_SIZE;

            item2.x = 14.5 * TILE_SIZE;
            item2.y = 11.5 * TILE_SIZE;

            item3.x = 9.5 * TILE_SIZE;
            item3.y = 9.5 * TILE_SIZE;

            levelMessage.SetText("To beat each level,\nyou will have to\ncomplete the ritual.");
            statusText.CenterOn(9.5 * TILE_SIZE, 14.5 * TILE_SIZE);
        }

        ///
        private function teachRitualPartTwoLevel():void
        {
            var w:Number = Settings.StageWidth;
            var h:Number = Settings.StageHeight;

            // left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0,
                0,
                TILE_SIZE,
                h );

            // right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                w - TILE_SIZE,
                0,
                TILE_SIZE,
                h );

            // floor
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                TILE_SIZE,
                10 * TILE_SIZE,
                w - 2 * TILE_SIZE,
                4 * TILE_SIZE );
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                TILE_SIZE,
                14 * TILE_SIZE,
                10 * TILE_SIZE,
                TILE_SIZE );

            // ceiling
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                TILE_SIZE,
                0,
                w - (2.0 * TILE_SIZE),
                TILE_SIZE );

            // floating platform
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                7 * TILE_SIZE,
                7 * TILE_SIZE,
                5 * TILE_SIZE,
                1 * TILE_SIZE);

            player.x = 3.5 * TILE_SIZE;
            player.y = 9.5 * TILE_SIZE;

            item2.x = 14.5 * TILE_SIZE;
            item2.y = 8.5 * TILE_SIZE;

            item1.x = 16.5 * TILE_SIZE;
            item1.y = 7.5 * TILE_SIZE;

            item3.x = 12.5 * TILE_SIZE;
            item3.y = 9.5 * TILE_SIZE;

            levelMessage.SetText("The order is always\n       blue,\n      yellow,\n        red");
            statusText.CenterOn(14 * TILE_SIZE, 14.5 * TILE_SIZE);
        }

        ///
        private function teachDoubleJumpLevel():void
        {
            var t:Number = TILE_SIZE;

            game.FollowTarget = player;

            // left wall 1
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                -10 * t,
                1 * t,
                10 * t,
                4 * t );

            // left wall 2
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                -20 * t,
                5 * t,
                27 * t,
                50 * t );

            // right wall 1
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                19 * t,
                1 * t,
                21 * t,
                4 * t );

            // right wall 2
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                12 * t,
                5 * t,
                31 * t,
                50 * t );

            // spikes!
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                7 * t,
                8 * t,
                5 * t );

            // under the spikes!
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                7 * t,
                9 * t,
                5 * t,
                1 * t );
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                7 * t,
                11 * t,
                5 * t,
                44 * t );

            player.x = 1.5 * TILE_SIZE;
            player.y = 1.5 * TILE_SIZE;

            item1.x = 3.5 * TILE_SIZE;
            item1.y = 2.5 * TILE_SIZE;

            item2.x = 9.5 * TILE_SIZE;
            item2.y = 4.5 * TILE_SIZE;

            item3.x = 15.5 * TILE_SIZE;
            item3.y = 2.5 * TILE_SIZE;

            levelMessage.CenterOn(1.5 * t, -0.5 * t);
            levelMessage.SetText("Keep pressing the jump key\nto jump again in the air!");

            statusText.CenterOn(9.5 * t, 10.5 * t);
        }

        //
        private function teachFourTimes():void
        {
            var t:Number = TILE_SIZE;

            game.FollowTarget = player;

            // left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0 * t,
                0 * t,
                1 * t,
                30 * t );

            // right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                17 * t,
                0 * t,
                1 * t,
                29 * t );

            // ceil
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                0 * t,
                16 * t,
                1 * t );

            // floor 1
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                29 * t,
                9 * t,
                1 * t );

            // floor 2
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                9 * t,
                28 * t,
                9 * t,
                1 * t );

            // platform1
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                10 * t,
                18 * t,
                8 * t,
                1 * t );

            // platform2
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                7 * t,
                7 * t,
                4 * t,
                1 * t );
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                6 * t,
                2 * t,
                1 * t,
                6 * t );

            player.x = 4.5 * TILE_SIZE;
            player.y = 11.5 * TILE_SIZE;

            item1.x = 15.5 * TILE_SIZE;
            item1.y = 3.5 * TILE_SIZE;

            item2.x = 14.5 * TILE_SIZE;
            item2.y = 3.5 * TILE_SIZE;

            item3.x = 13.5 * TILE_SIZE;
            item3.y = 27.5 * TILE_SIZE;

            levelMessage.CenterOn(9.5 * t, 24.5 * t);
            levelMessage.SetText("After jumping off the\n" +
                                 "ground,  you can jump\n" +
                                 " another three times\n" +
                                 "     in the air!\n\n" +
                                 " This can get you up\n" +
                                 "     really high!");

            statusText.CenterOn(14 * t, 29.5 * t);
        }

        ///
        private function beQuickLevel():void
        {
            var t:Number = TILE_SIZE;

            // left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0 * t,
                0 * t,
                1 * t,
                15 * t );

            // right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                17 * t,
                0 * t,
                1 * t,
                15 * t );

            // ceil
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                0 * t,
                16 * t,
                1 * t );

            // floor 1
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                14 * t,
                10 * t,
                1 * t );

            // floor 2
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                10 * t,
                13 * t,
                7 * t,
                1 * t );

            player.x = 9.5 * TILE_SIZE;
            player.y = 1.5 * TILE_SIZE;

            item1.x = 7.5 * TILE_SIZE;
            item1.y = 11.5 * TILE_SIZE;

            item2.x = 11.5 * TILE_SIZE;
            item2.y = 12.5 * TILE_SIZE;

            item3.x = 9.5 * TILE_SIZE;
            item3.y = 10.5 * TILE_SIZE;

            levelMessage.CenterOn(9.5 * t, 5.5 * t);
            levelMessage.SetText("Think fast!");

            statusText.CenterOn(14 * t, 14.5 * t);
        }

        ///
        private function trickyLevel():void
        {
            var t:Number = TILE_SIZE;

            // left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0 * t,
                0 * t,
                1 * t,
                15 * t );

            // right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                17 * t,
                0 * t,
                1 * t,
                7 * t );

            // ceil
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                0 * t,
                16 * t,
                1 * t );

            // floor 1
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                12 * t,
                10 * t,
                1 * t );

            player.x = 4.5 * TILE_SIZE;
            player.y = 11.5 * TILE_SIZE;

            item1.x = 16.5 * TILE_SIZE;
            item1.y = 2.5 * TILE_SIZE;

            item2.x = 13.5 * TILE_SIZE;
            item2.y = 4.5 * TILE_SIZE;

            item3.x = 13.5 * TILE_SIZE;
            item3.y = 8.5 * TILE_SIZE;

            levelMessage.CenterOn(7.5 * t, 5.5 * t);
            levelMessage.SetText("");

            statusText.CenterOn(14 * t, 14.5 * t);
        }

        //
        private function pitLevel():void
        {
            var t:Number = TILE_SIZE;

            // left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0 * t,
                0 * t,
                1 * t,
                15 * t );

            // right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                17 * t,
                0 * t,
                1 * t,
                15 * t );

            // ceil
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                0 * t,
                16 * t,
                1 * t );

            // left platform
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                5 * t,
                5 * t,
                10 * t );
            // right platform
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                13 * t,
                5 * t,
                6 * t,
                10 * t );

            // spikes 1
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                6 * t,
                13 * t,
                7 * t);
            // pit floor
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                6 * t,
                14 * t,
                7 * t,
                1 * t);

            player.x = 2.5 * TILE_SIZE;
            player.y = 4.5 * TILE_SIZE;

            item1.x = 7.5 * TILE_SIZE;
            item1.y = 11.5 * TILE_SIZE;

            item2.x = 11.5 * TILE_SIZE;
            item2.y = 11.5 * TILE_SIZE;

            item3.x = 9.5 * TILE_SIZE;
            item3.y = 11.5 * TILE_SIZE;

            levelMessage.CenterOn(9.5 * t, 3.5 * t);
            levelMessage.SetText("");

            statusText.CenterOn(14 * t, 1.5 * t);
        }

        //
        private function goUnderLevel():void
        {
            var t:Number = TILE_SIZE;

            // left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0 * t,
                0 * t,
                1 * t,
                15 * t );

            // right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                17 * t,
                0 * t,
                1 * t,
                15 * t );

            // ceil
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                0 * t,
                16 * t,
                1 * t );

            // left platform
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                14 * t,
                5 * t,
                1 * t );
            // right platform
            /*GameContent.addPlatform(
                game.Layers.PlatformLayer,
                13 * t,
                13 * t,
                6 * t,
                1 * t );*/

            // spikes 1
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                6 * t,
                9 * t,
                7 * t);
            // pit floor
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                6 * t,
                10 * t,
                7 * t,
                1 * t);

            player.x = 2.5 * TILE_SIZE;
            player.y = 4.5 * TILE_SIZE;

            item1.x = 7.5 * TILE_SIZE;
            item1.y = 11.5 * TILE_SIZE;

            item2.x = 11.5 * TILE_SIZE;
            item2.y = 11.5 * TILE_SIZE;

            item3.x = 9.5 * TILE_SIZE;
            item3.y = 11.5 * TILE_SIZE;

            levelMessage.CenterOn(9.5 * t, 2.5 * t);
            levelMessage.SetText("");

            statusText.CenterOn(14 * t, 14.5 * t);
        }

        //
        private function tunnelLevel():void
        {
            var t:Number = TILE_SIZE;

            // left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0 * t,
                0 * t,
                1 * t,
                15 * t );

            // right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                17 * t,
                0 * t,
                1 * t,
                15 * t );

            // ceil
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                0 * t,
                16 * t,
                1 * t );

            // left platform
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                14 * t,
                10 * t,
                1 * t );
            // right platform
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                13 * t,
                13 * t,
                4 * t,
                1 * t );

            // spikes 1
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                6 * t,
                9 * t,
                7 * t);
            // raised section
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                6 * t,
                10 * t,
                7 * t,
                4 * t);
            // lowceiling
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                6 * t,
                1 * t,
                7 * t,
                4 * t);

            // spikes 1
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                13 * t,
                12 * t,
                4 * t);

            player.x = 2.5 * TILE_SIZE;
            player.y = 4.5 * TILE_SIZE;

            item1.x = 7.5 * TILE_SIZE;
            item1.y = 7.5 * TILE_SIZE;

            item2.x = 8.5 * TILE_SIZE;
            item2.y = 8.5 * TILE_SIZE;

            item3.x = 16.5 * TILE_SIZE;
            item3.y = 6.5 * TILE_SIZE;

            levelMessage.CenterOn(9.5 * t, 2.5 * t);
            levelMessage.SetText("");

            statusText.CenterOn(14 * t, 14.5 * t);
        }

        //
        private function underOverLevel():void
        {
            var t:Number = TILE_SIZE;

            game.FollowTarget = player;

            // tunnel - left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                0 * t,
                10 * t,
                1 * t,
                4 * t );

            // tunnel - floor
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                13 * t,
                8 * t,
                1 * t );
            // tunnel - ceil
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                1 * t,
                10 * t,
                6 * t,
                1 * t );

            // pit 1 - left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                8 * t,
                14 * t,
                1 * t,
                9 * t );
            // pit 1 - ceil
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                6 * t,
                9 * t,
                8 * t,
                1 * t );
            // pit 1 - floor
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                9 * t,
                22 * t,
                7 * t,
                1 * t );
            // pit 1 - spikes
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                9 * t,
                21 * t,
                7 * t );
            // pit 1 - right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                16 * t,
                17 * t,
                1 * t,
                6 * t );

            // meanspike
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                9 * t,
                13 * t,
                1 * t );
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                9 * t,
                14 * t,
                1 * t,
                1 * t );

            // go under this
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                13 * t,
                10 * t,
                1 * t,
                4 * t );

            // post-pit platform
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                17 * t,
                17 * t,
                1 * t,
                1 * t );

            // pit 2 - left wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                18 * t,
                17 * t,
                1 * t,
                4 * t );
            // pit 2 - spikes
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                19 * t,
                19 * t,
                30 * t);
            // pit 2 - floor
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                19 * t,
                20 * t,
                30 * t,
                1 * t);
            // pit 2 - ceil
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                14 * t,
                10 * t,
                35 * t,
                1 * t);
            // pit 2 - right wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                49 * t,
                10 * t,
                1 * t,
                11 * t);
            // pit 2 - platform
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                48 * t,
                16 * t,
                1 * t,
                1 * t);


            // go under this, too
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                34 * t,
                10 * t,
                1 * t,
                4 * t );

            player.x = 1.5 * TILE_SIZE;
            player.y = 12.5 * TILE_SIZE;

            item1.x = 13.5 * TILE_SIZE;
            item1.y = 15.5 * TILE_SIZE;

            item2.x = 48.5 * TILE_SIZE;
            item2.y = 14.5 * TILE_SIZE;

            item3.x = 13.5 * TILE_SIZE;
            item3.y = 18.5 * TILE_SIZE;

            levelMessage.CenterOn(7.5 * t, 5.5 * t);
            levelMessage.SetText("");

            statusText.CenterOn(3.5 * t, 14.5 * t);
        }

        private function youWinLevel():void
        {
            var t:Number = TILE_SIZE;
            levelMessage.CenterOn(7.5 * t, 5.5 * t);

            game.FollowTarget = player;

            // floor 1
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                -7 * t,
                8 * t,
                21 * t,
                2 * t );
            // wall
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                11 * t,
                10 * t,
                2 * t,
                10 * t );

            // spikes
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                -20 * t,
                19 * t,
                31 * t );
            GameContent.addSpikes(
                game.Layers.PlatformLayer,
                13 * t,
                19 * t,
                30 * t );

            // floor below spikes
            GameContent.addPlatform(
                game.Layers.PlatformLayer,
                -20 * t,
                20 * t,
                63 * t,
                1 * t );


            player.x = 9.5 * TILE_SIZE;
            player.y = 7.5 * TILE_SIZE;

            item1.x = 18.5 * TILE_SIZE;
            item1.y = 12.5 * TILE_SIZE;

            item2.x = 13.5 * TILE_SIZE;
            item2.y = 11.5 * TILE_SIZE;

            item3.x = 9.5 * TILE_SIZE;
            item3.y = 11.5 * TILE_SIZE;

            levelMessage.CenterOn(7.5 * t, 3.5 * t);
            levelMessage.SetText("  Last Level!\n" +
                                 "You can do it!" );

            statusText.CenterOn(7.5 * t, 10.5 * t);
        }
    }
}
