﻿package compo_ritual.gamestates
{
    import compo_ritual.GameContent;
    import compo_ritual.Item;
    import compo_ritual.PlatformMan;
    import compo_ritual.RitualLevels;
    import flash.display.Bitmap;
    import flash.display.BitmapData;
    import flash.display.GradientType;
    import flash.display.InterpolationMethod;
    import flash.display.SpreadMethod;
    import flash.events.Event;
    import flash.geom.Matrix;
    import flash.geom.Point;
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import meyermike.collision.CollisionEvent;
    import meyermike.constants.Colors;
    import meyermike.constants.Keyboard;
    import meyermike.core.GameInterface;
    import meyermike.core.GameObject;
    import meyermike.core.GameState;
    import meyermike.InputUtil;
    import meyermike.PlatformerCharacter;
    import meyermike.script.step.Condition;
    import meyermike.script.step.Loop;
    import meyermike.script.step.RunFunc;
    import meyermike.Settings;
    import meyermike.Sounds;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Game extends GameState
    {
        private static const CAMSHAKE_THRESHOLD:Number = 699; // minimum collision speed for cam shake
        private static const CAMSHAKE_AMOUNT:Number = 3.0;
        private static const CAM_SHAKE_DECAY:Number = 10.0;
        private static const LAND_SOUND_THRESHOLD:Number = 100;

        private var levelUtil:RitualLevels;

        /**
         * create scene
         */
        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            levelUtil = new RitualLevels(game);
            levelUtil.begin();
            levelUtil.addEventListener( RitualLevels.EVENT_WIN, win);
            levelUtil.addEventListener( RitualLevels.EVENT_LOSE, lose);

            levelUtil.player.addEventListener(CollisionEvent.EVENT_COLLIDE, PlayerHitSomething);

            game.Music.StartNewMusic();
        }

        /**
         * when the player hits certain things, we need to make stuff happen
         */
        private function PlayerHitSomething(e:CollisionEvent):void
        {
            if (!(e.object is Item))
            {
                var a:GameObject = e.target as GameObject;
                var b:GameObject = e.object;

                var xspeed:Number = (b.dx - a.dx) * e.collisionInfo.normal.x;
                var yspeed:Number = (b.dy - a.dy) * e.collisionInfo.normal.y;

                var speed:Number = Math.sqrt( (xspeed * xspeed) + (yspeed * yspeed) );
                if (speed > CAMSHAKE_THRESHOLD)
                {
                    Game.CameraShake = CAMSHAKE_AMOUNT;
                    GameContent.PlaySound_Collide();
                }
                else if ( (e.collisionInfo.normal.y < 0) && ( a.dy > (LAND_SOUND_THRESHOLD)) )
                {
                    // landing! play a step sound if we didn't already do a crash sound
                    GameContent.PlaySound_Footstep();
                }
            }
        }

        /**
         * level won
         */
        private function win(e:Event = null):void
        {
        }

        /**
         * level lost
         */
        private function lose(e:Event = null):void
        {
            Game.CameraShake = CAMSHAKE_AMOUNT;
            GameContent.PlaySound_Collide();
        }

        /**
         * Clean up!
         */
        public override function End():void
        {
            super.End();
            Game.Layers.RemoveAll();
            Game.CameraShake = 0;
        }

        /**
         * per-frame behavior
         */
        public override function Update():void
        {
            super.Update();

            levelUtil.update(Game.Time.TimeDelta);

            // if the camera is shaking, let's slow it down
            if (Game.CameraShake > 0.0)
            {
                var delta:Number = CAM_SHAKE_DECAY * Game.Time.TimeDelta;
                if (Game.CameraShake < delta)
                {
                    Game.CameraShake = 0;
                }
                else
                {
                    Game.CameraShake -= delta;
                }
            }
        }
    }
}
