﻿package meyermike.util
{
    import flash.geom.Point;
    import flash.geom.Rectangle;
    /**
     * Math-related utility functions
     * @author Mike Meyer
     */
    public class MathUtil
    {
        /**
         * Do a linear interpolation between two values.  t in [0,1]
         */
        public static function Lerp(a:Number, b:Number, t:Number):Number
        {
            return a + (t*(b-a));
        }

        /**
         * Keep a value within the range
         */
        public static function Clamp(value:Number, min:Number, max:Number):Number
        {
            return Math.max( min, Math.min( max, value ) );
        }

        /**
         * Wrap a value around to keep it within a range
         */
        public static function Wrap(value:Number, min:Number, max:Number):Number
        {
            if ( (max - min) == 0 )
            {
                return min;
            }

            // deal with the edge cases
            if (max < min)
            {
                var temp:Number = max;
                max = min;
                min = temp;
            }

            var wrapped:Number = ( (value-min) % (max - min)) + min;
            if (wrapped < min) { wrapped += (max - min); }
            return wrapped;
        }

        /**
         * given a rect, returns a new point telling you where its
         * center is
         */
        public static function GetRectCenter(rect:Rectangle):Point
        {
            return new Point(
                0.5*(rect.left+rect.right),
                0.5*(rect.top+rect.bottom) );
        }

        /**
         * given an x and y, determines the slope of a line
         * through (0,0) and that point.
         * MAY RETURN INFINITY (or negative infinity)
         * if you give it (0,0), it's obviously not going
         * to be able to give you a slope
         */
        public static function GetSlope(x:Number, y:Number):Number
        {
            if (0.0 != x)
            {
                return (y / x);
            }
            else if (0.0 > y)
            {
                return Number.NEGATIVE_INFINITY;
            }
            else
            {
                return Number.POSITIVE_INFINITY;
            }
        }

        /**
         * returns the "sign" of a number
         * n > 0  ==> 1
         * n < 0  ==> -1
         * n = 0  ==> 0
         */
        public static function Sign(n:Number):Number
        {
            if (n > 0) return 1;
            if (n < 0) return -1;
            return 0;
        }

        /**
         * return true if the numbers are both negative or both positive
         * if either is zero, this returns false
         */
        public static function SameSign(x:Number, y:Number):Boolean
        {
            return ((x*y)>0.0);
        }

        /**
         * return true if one number is negative and the other positive
         * if either is zero, this returns false
         */
        public static function OppositeSign(x:Number, y:Number):Boolean
        {
            return ((x*y)<0.0);
        }

        /**
         * Scale a point/vector and return it (leaves the original alone)
         */
        public static function ScaleVector(v:Point, scale:Number):Point
        {
            return new Point(v.x * scale, v.y * scale);
        }

        /**
         * move at most maxDist> units from <a> in the direction of <b>,
         * stopping at <b> if it's enough to take us there
         */
        public static function MoveToward(a:Number, b:Number, maxDist:Number):Number
        {
            if ( (b - a) < maxDist )
            {
                return b;
            }
            else if (b > a)
            {
                return a + maxDist;
            }
            else
            {
                return a - maxDist;
            }
        }

        /**
         * move at most <maxDist> units from <a> in the direction of <b>,
         * stopping at <b> if it's enough to take us there
         */
        public static function MoveToward2d(a:Point, b:Point, maxDist:Number):Point
        {
            var distance:Number = Point.distance(a, b);

            if (distance < maxDist)
            {
                return b;
            }
            else
            {
                var delta:Point = b.subtract(a);
                delta.normalize(maxDist);

                return a.add(delta);
            }
        }
    }

}
